/*
 * example_basic.c
 *
 * demonstrates simple operation of the libri library to communicate with
 * a DPD80 photodetector.
*/

// libri includes
#include "ri.h"

// for uint16_t types
#include "stdint.h"

//for printf
#include "stdio.h"

int main(int argc, char *argv[])
{
    // variable to check the return status of libri functions.
    // functions return 0 on success and non zero when an error
    // occurs.
    int err = 0;

    // call this function once to initialize the library.
    // when finished, call ri_exit() to uninitialize the library.
    ri_init();

    // references to the DPD80 are stored as an opaque pointer with the type ri_device*
    ri_device* device = 0;

    // try and open the first available device.
    device = ri_open_device();
    if (device == 0) {
        printf("unable to find a ri device\n");
    }
	else {

		// simple features can be set using the ri_set functions.
		// for example, lets turn off highgain mode (turn on lowgain mode)
		ri_set_highgain(device, 0);

		// now let's collect data from the photodetector.
		// we need a buffer to store the data in.  we are going to use the
		// ri_get_raw_data function which returns samples exactly as the adc
		// reads them, packed into unsigned 16 bit integers.
		// the digital inputs,  sync and trig, are stored in the highest 2 bits,
		// so when reading the samples later we zero those two bits with "& 0x3fff".
		uint16_t buffer[1024];

		// now call ri_get_raw_data to collect 1024 samples
		err = ri_get_raw_data(device, 1024, buffer);
		if (err) {
			printf("data collection failed.\n");
            ri_close_device(device);
            ri_exit();
            return 0;
		}

		printf("first 10 samples (ADC values):\n");
		for (int i = 0; i < 9; i++)
			printf("%d, ", buffer[i] & 0x3fff);
		// print 9th value seperatly to remove trailing comma
		printf("%d\n", buffer[9] & 0x3fff);


        // each DPD80 contains a calibration from adccode to uW stored on the
        // device. This calibration can be retreived with the ri_get_value function.

        float calibration[2] = {0., 0.};
        err = ri_get_value(device, RI_CAL_DIGITAL_HIGHGAIN, calibration);
        if (err) {
            printf("error retreiving calibration\n");
            ri_close_device(device);
            ri_exit();
            return 0;
        }


        printf("first 10 samples (uW):\n");
        for (int i = 0; i < 9; i++)
            printf("%f, ", (buffer[i] & 0x3fff)*calibration[0] + calibration[1]);
        // print 9th value seperatly to remove trailing comma
        printf("%f\n", (buffer[9] & 0x3fff)*calibration[0] + calibration[1]);

        float mean = 0;
        for (int i = 0; i < 1024; i++)
            mean += (buffer[i] & 0x3fff)*calibration[0] + calibration[1];
        mean = mean / 1024.;
        printf("mean power: %f uW\n", mean);

		// close the device;
		device = ri_close_device(device);
	}

    // uninitialize the library
    ri_exit();

#ifdef _MSC_VER
	// if Visual Studio get keyboard input at end of program so it doesn't automatically close
	getchar();
#endif
    return 0;
}
